/***************************************************************************/
/*                                                                         */
/*  TempLog.c                                                              */
/*             Temperatur Logger                                           */
/*             PIC16F84  4Mhz                                              */
/*                                                                         */
/* Interface :                                                             */
/*           Red   LED = Port A 3                                          */
/*           Green LED = Port A 2                                          */
/*                  Rx = Port A 1                                          */
/*                  Tx = Port A 0                                          */
/*     PC/Mode Switch  = Port B 5                                          */
/*       Start Switch  = Port B 2                                          */
/*   24LC16 DS1621 SDA = Port B 1                                          */
/*   24LC16 DS1621 SCL = Port B 0                                          */
/*                                                                         */
/* (c) Matthias Stehr 15.01.2002                                           */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
#include <16F84.h>
#use delay(clock=4000000)          // Crystal on the realer Hardware 4 Mhz
#use i2c(MASTER, SDA=PIN_B1, SCL=PIN_B0)
#use rs232(baud=9600, xmit=PIN_A1, rcv=PIN_A0)

void temp_config(byte data);
void init_temp();
void read_temp();
void delay_s(byte s);
void rs232_prog();
BYTE read_ext_eeprom(long int address);
void write_ext_eeprom(long int address, byte data);
void initEEProm(void);
void readEEProm(void);
void print_data(void);

#define PortAConfig  0b00000001   /* Configure port A  0 = output       */
#define EE_SEC       0x00         // Addr. in EE-Prom to store interval


byte datah,datal;
byte sec;
short store;
long index;
long int j;

/**************************************************************************/
/*                                                                        */
/**************************************************************************/
void main()
{
   set_tris_a( PortAConfig ); // Set Port Direction
   init_temp();               // Init DS1621 Temperature Sensor
   sec = read_eeprom(EE_SEC); // Read interval from internal EE-Prom
   store = FALSE;             // Logging not active
   index = 0;
   // Go into endless loop
   do{
      output_high(PIN_A2);    // Green LED on
      read_temp();            // Get temperature from DS1621
      print_data();           // Sent data to RS232 port
      if(store)               // If TRUE write data to EE-Prom 
      {
         output_high(PIN_A3);             // Red LED on
         write_ext_eeprom(index,datah);   // MSB
         index++;
         write_ext_eeprom(index,datal);   // LSB
         index++;
         output_low(PIN_A3);              // Red LED off
      }
      if(index == 2048)       // EE-Prom is full stop logging
         store = FALSE;
      output_low(PIN_A2);     // Green LED off
      delay_s(sec);           // Wait a little bit and check the input
      }
    while(1);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/
void temp_config(byte data) {
   i2c_start();       /* send start I2C Start bit */
   i2c_write(0x90);   /* send ADDRESS byte        */
   i2c_write(0xac);   /* send COMMAND byte Write to Configuration Register*/
   i2c_write(data);   /* write data to Register   */
   i2c_stop();        /* send stop bit            */
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/
void init_temp() {
	/* set SDA SCL to 5 V */
   output_high(PIN_B1);
   output_high(PIN_B0);

   i2c_start();       /* send start I2C Start bit */
   i2c_write(0x90);   /* send ADDRESS byte        */
   i2c_write(0xee);   /* send COMMAND byte Start Convert */
   i2c_stop();        /* send stop bit            */
   temp_config(8);    /* set to continuously conversion mode */
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/
void read_temp() {        
   i2c_start();        /* send start I2C Start bit */
   i2c_write(0x90);    /* send ADDRESS byte        */
   i2c_write(0xaa);    /* send COMMAND byte Read Temperature */
   i2c_start();        /* send start I2C Start bit */
   i2c_write(0x91);    /* send ADDRESS byte with Read bit    */
   datah=i2c_read();   /* read temperature high byte */
   datal=i2c_read(0);  /* read temperature low byte  */
   i2c_stop();         /* send stop bit              */
   if((datah&0x80)!=0) /* only positive temperature are handled */
      datah = 0;       /* in case of minus valus temperature is set to zero */
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/
void delay_s(byte s)
{
   byte i;
   for (i = 0; i < s; i++)
   {
      delay_ms(1000);
	  // PC/Mode switch 
	  // The RS232 port is not controlled by an interrupt, to read data from the PC
	  // the PIC must be set to Listen (PC) mode. This is done with a switch on port B5
      if(!input(PIN_B5))      
      {
		  // Now PIC is in PC mode and can be controled with the PC program
         output_high(PIN_A3); // Red LED on
         rs232_prog();        // Read data from PC
         i=s;                 // Break For-Loop
      }
	  // Check if the Start switch was pushed
      if(!input(PIN_B2))      // START switch
      {
         index = 0;
         store = TRUE;        // Set recording active bit
         initEEProm();        // Set EE-Prom Data to zero    
      }         
   }
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/
void rs232_prog()
{
   char c;  
   c = getch();               // Read command byte from PC
   switch (c)
   {
      case 0: sec = getch();              // Read interval from PC, this value
		                                  // defines the measurement interval in sec.
              write_eeprom(EE_SEC,sec);   // save data in EE prom 
         break;
      case 1: readEEProm();               // send logged data to PC
         break;
      default: ;                          // not used, do nothing
         break;
   }
   output_low(PIN_A3);        // Red LED off   
}




/**************************************************************************/
/*                                                                        */
/**************************************************************************/
byte read_ext_eeprom(long int address) {
   byte data;

   i2c_start();
   i2c_write((0xa0|(byte)(address>>7))&0xfe);
   i2c_write(address);
   i2c_start();
   i2c_write((0xa0|(byte)(address>>7))|1);
   data=i2c_read(0);
   i2c_stop();
   return(data);
}



/**************************************************************************/
/*                                                                        */
/**************************************************************************/
void write_ext_eeprom(long int address, byte data) {

   i2c_start();
   i2c_write((0xa0|(byte)(address>>7))&0xfe);
   i2c_write(address);
   i2c_write(data);
   i2c_stop();
   delay_ms(11);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/
void readEEProm(void)
{
  // read the complete content of the I2C EE-prom
  // and send it to the RS232 port.
  j=0;
  while(TRUE)
   {
   datah = read_ext_eeprom(j);
   j++;
   datal = read_ext_eeprom(j);
   print_data();        // Sent data to PC
   if(j == 2047)
      {
	  // Last byte was sent,
      putchar(0xFE);    // Sent end of data information
      putchar(0xFE);
      putchar(0xFE);
      break;
      }
   ++j;
   }
}



/**************************************************************************/
/*                                                                        */
/**************************************************************************/
void initEEProm(void)
{
   output_high(PIN_A2);     // Green LED on
   j=0;
   while(TRUE)
   {
      output_high(PIN_A3);  // Red LED on
      write_ext_eeprom(j,0x00);
      output_low(PIN_A3);   // Red LED off
      if(j == 2047)
         break;
      ++j;
   }
   output_low(PIN_A2);      // Green LED off
}


/**************************************************************************/
/*                                                                        */
/**************************************************************************/
void print_data(void)
{
   // send the temperature information to the RS232 port
   // if a PC is connected, the data can be displayed or not
   putchar(0xFF);    // First Command 0xFF is sent 
                     // This byte can be used as a trigger on the PC, to indicate that
                     // 2 bytes temperature are following
   putchar(datah);
   putchar(datal);
}